-- mcSara
-- micro version of SARA, a recombinant music program
-- david cuny
-- dcuny@lanset.com

constant VERSION     = "microSara v2.1"



-- TO DO:
--  atn - current version is buggy
--  transposition - need to include key changes
--  splice cadences using cope's code
--  embellishment
--  ties

include sort.e
include wildcard.e
include dialog.e
     
with trace     
--without trace
without warning
without type_check

-- use graphic font
graphic_font()


-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- General Tools

-----------------------------------------------------------------------------
function first( sequence s )

    -- returns first element in list
    --  ? first({1,2,3})
    --  --> 1
    --
    --  ? first({})
    --  --> {}
    
    if length( s ) then
        return s[1]
    else
        return {}
    end if
end function

-----------------------------------------------------------------------------
function rest( sequence s )

    -- returns everything but first element in list
    --  ? rest({1,2,3})
    --  --> {2,3}
    --
    --  ? rest({})
    --  --> {}
    
    if length( s ) > 1 then
        return s[2..length(s)]
    else
        return {}
    end if
end function

-----------------------------------------------------------------------------
function null( sequence s )
    -- returns true if list is empty
    --  ? null({})
    --  --> 1
    --
    --  ? null({1,2,3})
    --  --> 0
    
    return length(s) = 0
    
end function


-----------------------------------------------------------------------------
function equal( object o1, object o2 )

    -- returns 1 if objects are equal, else 0
    --  ? equal( 2, 12 )
    --  --> 0
    
    if compare( o1, o2 ) then
        return 0
    else
        return 1
    end if
    
end function

-----------------------------------------------------------------------------
function max( sequence list )

    -- returns highest item in list
    --  ? max( {4,4,5,4,2,4,3,2,1,1,0,4,2,2,4,2,2,0,1,4})
    --  --> 5

    integer maxAt

    -- initialize
    maxAt = 1
    
    for i = 2 to length( list ) do
        if list[i] > list[maxAt] then
            maxAt = i
        end if
    end for
    
    return list[maxAt]
    
end function    

-----------------------------------------------------------------------------
function count( object item, sequence list)
    
    -- returns number of times item appears in list
    --  ? count( 5, {1,3,5,7,5,6} )
    --  --> 2
    
    integer sum
    sum = 0
    for i = 1 to length( list ) do
        if equal( item, list[i] ) then
            sum = sum + 1
        end if
    end for
    
    return sum
    
end function


-----------------------------------------------------------------------------
function word( integer index, sequence s )

    -- returns nth word from string
    -- ex:  word( 2, "this is text" )
    --      --> "is"
    
    integer count, start
    
    -- add padding
    s = s & " "
    
    start = 1
    count = 0
    for i = 1 to length( s ) do
        if s[i] = ' ' then
            count = count + 1
            if count = index then
                return s[start..i-1]
            else
                start = i+1
            end if
        end if
    end for
end function    

-----------------------------------------------------------------------------
function chooseOne( sequence s )
    
    -- return random selection from s
    -- ex:  chooseOne( {1,2,3} )
    --      --> 2
    if length( s ) then
        return s[rand(length(s))]
    else        
        -- return empty list
        return s
    end if
    
end function


-----------------------------------------------------------------------------
function abs( atom a )
    
    -- return absolute value of number
    -- ex:  ? abs( -12 )
    --      --> 12
    
    if a < 0 then
        return -a
    else
        return a
    end if
    
end function 

-----------------------------------------------------------------------------
function diff( atom a, atom b )

    -- returns the difference between two numbers
    --  ex: ? diff( 3,7 )
    --  --> 4
    
    return abs( a-b )
    
end function

-----------------------------------------------------------------------------
function remove( object o, sequence s )

    -- remove all instances of object from list
    -- ex:  ? remove( 1, {3,2,1} )
    --      --> {3,2}
    
    integer at
    sequence new

    -- initialize
    new = {}

    for i = 1 to length( s ) do      
        -- if not the object...
        if compare( o, s[i] ) then
            -- ... then add to the new list
            new = append( new, s[i] )
        end if
    end for

    return new
    
end function


-----------------------------------------------------------------------------
function removeNth( sequence s, integer i )
    -- remove nth item from list
    return s[1..i-1] & s[i+1..length(s)]
end function


-----------------------------------------------------------------------------
function isEven( atom i )
    -- return true if the argument is even
    -- ex:  isEven( 8 )
    --      --> 1
    
    return not remainder( i, 2 )
    
end function


-----------------------------------------------------------------------------
function isOdd( atom i )
    -- return true if the argument is an odd number
    -- ex:  isOdd( 8 )
    --      --> 0
    
    return remainder( i, 2 )
    
end function

-----------------------------------------------------------------------------
function reverse( sequence list )
    -- reverse a list
    -- ex:  reverse( {1,2,3} )
    --      --> {3,2,1}
    sequence s                 
    s = {}
    for i = length( list ) to 1 by -1 do
        s = append( s, list[i] )
    end for
    return s
end function


     
--
-- DATABASE SUPPORT
--

-- match criteria
    integer            
        useAtn,     -- if true, uses ATN to generate phrase
        useMatn,    -- if true, uses MATN on each voice
        useStart,   -- if true, uses start note as part of matching
        useSeek,    -- if true, uses seek note as part of matching
        useWeak,    -- if true, only has to match first function of analysis
        changeMode, -- if true, change mode to minor
        matchSemi   -- allowable variation (in semitones) of start/seek

    sequence
        composer,   -- composer selected for composition
        fileName,   -- name of file to create
        form        -- for to use in creating file

-- defaults
        useAtn      = 0
        useMatn     = 1
        useStart    = 0
        useSeek     = 1
        useWeak     = 0
        changeMode  = 0
        matchSemi   = 3


constant
    -- phrase header
    Composer    = 1,    -- style of music,                  
    Style       = 2,    -- "rags", "maz", etc.
    Form        = 3,    -- "one", "two", "intro"
    Source      = 4,    -- where the material was from
    CadenceType = 5,    -- type of cadence    
    Number      = 6,    -- measure number
    Length      = 7,    -- measures of events
    Analysis    = 8,    -- analysis of the measure
    Starts      = 9,    -- starting notes of the phrase
    Events      = 10,   -- midi events
    NewMeasure  = repeat( 0, 10 ),
    
    -- midi event    
    Time        = 1,    -- ticks offset for MIDI event
    Pitch       = 2,    -- MIDI note
    Duration    = 3,    -- MIDI note duration
    Voice       = 4,    -- voice note belongs to
    Volume      = 5,    -- volume to play pitch at
    NewEvent    = repeat( 0, 5 ),

    -- measure duration
    TicksPerBeat = 1000, -- ticks per beat

    -- phrases returned from regenerate_phrase()
    Intro       = 1,    -- intro
    Half        = 2,    -- half cadence
    Full        = 3     -- full cadence

-- analysis symbols
global constant  
    s1  = 1,    p1 = 5,     e1 = 9,     a1 = 13,    c1 = 17,
    s2  = 2,    p2 = 6,     e2 = 10,    a2 = 14,    c2 = 18,
    s3  = 3,    p3 = 7,     e3 = 11,    a3 = 15,    c3 = 19,
    s4  = 4,    p4 = 8,     e4 = 12,    a4 = 16,    c4 = 20,
    
    speac = {   "s1", "s2", "s3", "s4",
                "p1", "p2", "p3", "p4",
                "e1", "e2", "e3", "e4",
                "a1", "a2", "a3", "a4",
                "c1", "c2", "c3", "c4" }


sequence
    db,                 -- musical database
    explain
    
    db = {}

integer theRecombinance         theRecombinance = 100


-----------------------------------------------------------------------------
-- Analysis

constant 

-- functions ordered to analysis lexicon
    C1  = 1,
    P1  = 2,
    A1  = 3,
    A2  = 4, 
    C4  = 5,
    P2  = 6,
    C2  = 7,
    S1  = 8,
    S3  = 9,
    E1  = 10,
    E3  = 11,
    C3  = 12,
    E2  = 13,
    E4  = 14,
    A3  = 15,
    P3  = 16,
    P4  = 17,
    S4  = 18,
    A4  = 19,
    S2  = 20,

-- names of functions
FUNCTION_NAME = { "C1","P1","A1","A2","C4","P2","C2","S1","S3","E1","E3",
                    "C3","E2","E4","A3","P3","P4","S4","A4","S2" },


-- notes associated with functions
ANALYSIS_LEXICON = {
    {24,36,48,60,72,84,96,108,28,40,52,64,76,88,100,31,43,55,67,79,91,103},
    {29,41,53,65,77,89,101,33,93,105,81,45,57,69,24,36,48,60,72,84,96,108},
    {31,43,55,67,79,91,103,35,47,59,71,83,95,107,26,38,50,62,74,86,98,29,41,53,65,77,89,101},
    {35,47,59,71,83,95,107,26,38,50,62,74,86,98,29,41,53,65,77,89,101},
    {28,40,52,64,76,88,100,31,43,55,67,79,91,103,35,47,59,71,83,95,107},
    {26,38,50,62,74,86,98,89,101,77,65,29,41,53,93,105,81,69,33,45,57},
    {33,45,57,69,81,93,105,24,36,48,60,72,84,96,108,28,40,52,64,76,88,100},
    {26,38,50,62,74,86,98,30,33,42,54,66,78,90,102,45,57,69,81,93,105},
    {28,40,52,64,76,88,100,32,35,44,56,68,80,92,104,47,59,71,83,95,107},
    {33,45,57,69,81,93,105,25,37,49,61,73,85,97,28,40,52,64,76,88,100},
    {35,47,59,71,83,95,107,27,39,51,63,75,87,99,30,42,54,66,78,90,102},
    {24,36,48,60,72,84,96,108,28,31,34,40,52,64,76,88,100,43,55,67,79,91,103,46,58,70,82,94,106},
    {25,37,49,61,73,85,97,28,31,34,40,52,64,76,88,100,43,55,67,79,91,103,46,58,70,82,94,106},   
    {27,39,51,63,75,87,99,24,30,33,36,48,60,72,84,96,108,42,54,66,78,90,102,45,57,69,81,93,105},
    {32,44,56,68,80,92,104,26,29,35,38,50,62,74,86,98,41,53,65,77,89,101,47,59,71,83,95,107},
    {32,44,56,68,80,92,104,24,30,36,48,60,72,84,96,108,42,54,66,78,90,102},
    {25,37,49,61,73,85,97,29,32,41,53,65,77,89,101,44,56,68,80,92,104},
    {30,42,54,66,78,90,102,25,37,49,61,73,85,97,34,46,58,70,82,94,106},
    {27,39,51,63,75,87,99,31,43,55,67,79,91,103,34,46,58,70,82,94,106},
    {34,46,58,70,82,94,106,26,38,50,62,74,86,98,29,41,53,65,77,89,101}}

-- these represent e and a naturals for use in change-mode
constant FLATTEN = {40, 52, 64, 76, 88, 45, 57, 69, 81, 93 }                     


-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- Analysis Tools

-----------------------------------------------------------------------------
function onTheBeat( atom beat )
    
    -- determines if the time is on or off the beat
    --  ? onTheBeat(18000)
    --  --> 1
    --
    --  ? onTheBeat(18001)
    --  --> 0
    
    return ( remainder( beat, TicksPerBeat ) = 0 )
    
end function


-----------------------------------------------------------------------------
function weightNote( sequence event )

    -- used by weight-notes it weights its argument by duration
    --  ? weightNote({0, 79, 250, 1, 100}) 
    --  {79}        
    --
    --  ? weightNote({0, 64, 1000, 2, 100}) 
    --  --> {64,64,64,64}
    
    integer weight

    if  onTheBeat( event[Time] )
    and event[Duration] > 500
    and event[Volume] > 100 then
        weight = 5
        
    elsif onTheBeat( event[Time] )
    and   event[Duration] > 500
    and   event[Volume] > 75 then
        weight = 4
        
    elsif event[Duration] > 500
    and   event[Volume] > 75 then
        weight = 3
        
    elsif event[Duration] > 250
    and   event[Volume] > 75 then
        weight = 2
        
    else
        weight = 1
    end if
    
    -- return weighted note
    return repeat( event[Pitch], weight )
    
end function    


-----------------------------------------------------------------------------
function weightNotes( sequence voice )
    
    -- used by analyze it weights its argument members by duration
    --  ? weightNotes({{0,79,250,1,100},{0,64,250,2,100},{0,48,500,3,100},
    --                {500,65,250,2,100},{500,50,250,3,100}} )
    --  --> {79,64,48,48,65,50}
        
    sequence weights
    
    weights = {}
    for i = 1 to length( voice ) do
        weights = weights & weightNote(voice[i])
    end for                                  
    
    return weights
    
end function

-----------------------------------------------------------------------------
function myCount( sequence list1, sequence list2)

    -- used by countHarmonicNotes it counts its first argument in its 
    -- second argument:
    --  ? myCount({79,64,48,48,65,65,50,50},
    --            {38,50,62,74,86,41,53,65,77,89,46,58,70,82,94})
    --  --> 4

    integer sum
    sum = 0   
    
    -- for each item...
    for i = 1 to length( list1 ) do
        -- how many times does it appear?
        sum = sum + count( list1[i], list2 )
    end for
           
    return sum
           
end function


-----------------------------------------------------------------------------
function countHarmonicNotes( sequence harmonicNotes, sequence lexicon )

    -- counts number of matches of harmonics against the lexicon
    --  ? countHarmonicNotes( {79,64,48,48,65,65,50,50}, ANALYSIS_LEXICON )
    --  --> {4,4,5,4,2,4,3,2,1,1,0,4,2,2,4,2,2,0,1,4}

    sequence sum
    
    -- initialize
    sum = repeat( 0, length( ANALYSIS_LEXICON ) )

    -- for each function
    for i = 1 to length( ANALYSIS_LEXICON ) do
        sum[i] = myCount( harmonicNotes, ANALYSIS_LEXICON[i] )
    end for
    
    return sum

end function


-----------------------------------------------------------------------------
function findPositionOfMax( sequence s )

    -- returns position of highest value in list
    -- ex:  ? findPositionOfMax( {1,1,2,1,3,1,3} )
    --      --> 5

    integer target          
    
    return find( max(s), s )

end function

-----------------------------------------------------------------------------
function getRelevantAnalyses( sequence counts, integer highest )

    -- used by compare it returns an appropriate index of possible analyses
    -- ex: ? getRelevantAnalyses( {2,2,2,1,1,0,2}, 2 )
    --      --> {1,2,3,7}

    sequence s
    s = {}
    for i = 1 to length( counts ) do
        if counts[i] = highest then
            s = append( s, i )
        end if
    end for
    
    return s
    
end function

-----------------------------------------------------------------------------
function findNearestFront( integer pitch, sequence lexicon, sequence relevant )

    -- return the index of the lexicon the pitch is the closest to the
    -- front of relevant lexicons. relevant contains a list of indexes of
    -- lexicons to consider
    -- ex:  findNearestFront( 2, {{1,2,3},{3,4,5},{2,4,5}}, {1,2,3} )
    --      --> 3

    integer pos, lex, at, this

    -- initialize
    lex = 0
    pos = 10000 -- (very *bad* match)
    
    -- for each lexicon
    for i = 1 to length( relevant ) do
        -- which lexicon is being considered?
        this = relevant[i]
        -- find position in lexicon
        at = find( pitch, lexicon[this] )
        -- in lexicon?
        if at then
            -- closer to front?
            if at < pos then
                -- set as current winner
                lex = this
                pos = at
            end if
        end if
    end for

    return lex
            
end function


-----------------------------------------------------------------------------
function getFunction( sequence beat )

    -- returns a SPEAC symbol for the analysis of its argument:
    --  ? getFunction( {{0,79,250,1,100},{0,64,250,2,100},{0,48,500,3,100},
    --                  {500,65,500,2,100},{500,50,500,3,100}} )
    --  --> 3

    integer highest, lowestPitch, best
    sequence weighted, counts, relevant

    -- duplicate the note n times, to give it more weight
    weighted = weightNotes( beat )
    
    -- count the matches against the analysis lexicon
    counts = countHarmonicNotes( weighted, ANALYSIS_LEXICON )
    
    -- get the highest score
    highest = max( counts )
    
    -- more than one that matches high score?
    if count( highest, counts ) then   
        -- get indexes of relevant lexicons
        relevant = getRelevantAnalyses( counts, highest )
        -- sort so the lowest pitch is considered first
        weighted = sort( weighted )    
        
        -- it's possible that the relevant analyses don't include the
        -- lowest pitch, so they all have to be considered         
        for i = 1 to length( weighted ) do
            best = findNearestFront( weighted[i], ANALYSIS_LEXICON, relevant )
            -- found a match?
            if best then   
                exit
            end if
        end for
    else                          
        -- return index of best lexicon
        best = find( highest, counts )
    end if

    return best
    
end function                                                

-----------------------------------------------------------------------------
function analyze( sequence beats )

    -- returns a SPEAC symbol for the analysis of its argument:
    --  ? analyze( {{{0,79,250,1,100},{0,64,250,2,100},{0,48,500,3,100},
    --                {500,65,500,2,100},{500,50,500,3,100}}} )
    --  --> {3}

    sequence analysis                    
    
    -- initialize
    analysis = repeat( 0, length( beats ) )

    -- for each beat
    for i = 1 to length( beats ) do
        -- determine the function            
        analysis[i] = getFunction( beats[i] )
    end for       

    return analysis
end function


-----------------------------------------------------------------------------
function breakIntoBeats( sequence events, integer size )

    -- breaks it's data into sized chunks
    --  ? breakIntoBeats(
    --          {{0,72,1000,1,100},{0,64,1000,2,100},{0,48,250,3,100},
    --          {250,60,250,3,100},{500,59,250,3,100},{750,60,250,3,100) . . .
    --  --> {{{0,72,1000,1,100},{0,64,1000,2,100},{0,48,250,3,100},
    --          {0,72,1000,1,100},{0,64,1000,2,100},{0,48,250,3,100} . . .

    integer fits
    sequence measures, measure, remains, event

    -- initialize
    measures = {}
    remains = events

    -- loop until no events remain    
    while 1 do   
    
        -- finished?
        if null( remains ) then 
            exit
        end if
        
        -- process remaining events
        events = remains
        remains = {}     
        measure = {}

        -- look at each event
        for i = 1 to length( events ) do              
        
            -- get an event
            event = events[i]
            
            -- start time falls within window?
            if event[Time] < size then
            
                -- need to break at the beat?
                if event[Time] + event[Duration] > size then

                    -- truncate time to fit window
                    fits = size - event[Time]
                    event[Duration] = fits
                    
                    -- store
                    measure = append( measure, event )
                    
                    -- restore event
                    event = events[i]
                    
                    -- remove amount played from duration
                    event[Time] = 0
                    event[Duration] = event[Duration] - fits
                    
                    -- place in pending
                    remains = append( remains, event )
                    
                else
                    -- place in measure as is
                    measure = append( measure, event )
                end if
                
                
            else
                -- subtract start time
                event[Time] = event[Time] - size
                
                -- place in pending
                remains = append( remains, event )
            end if
                        
        end for        
        
        -- accumulate measure
        measures = append( measures, measure )            
                
    end while

    return measures
    
end function



-----------------------------------------------------------------------------
procedure printAnalysis( sequence beats )
    -- debugging tool to test correctness of analyze function
    --  ex: printAnalysis(  {{0, 55, 1000, 3, 100}, 
    --                      {0, 67, 1000, 2, 100}, 
    --                      {0, 74, 1000, 1, 100}, 
    --                      {1000, 63, 1000, 3, 100}, 
    --                      {1000, 66, 2000, 2, 100}, 
    --                      {1000, 72, 750, 1, 100}, 
    --                      {1750, 83, 250, 1, 100}, 
    --                      {2000, 55, 1000, 3, 100}, 
    --                      {2000, 81, 1000, 1, 100}} )
    --      --> {A1, E4, S1}

    sequence s
    
    -- break into beats and analyze
    s = analyze( breakIntoBeats(beats, 1000 ) )
    
    -- display results
    puts( 1, "{" )
    for i = 1 to length( s ) do
        -- convert element to function name
        puts( 1, FUNCTION_NAME[s[i]] )
        if i < length( s ) then
            puts( 1, ", " )
        end if
    end for
    puts( 1, "}\n" )
    
end procedure


-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- Database Tools


-----------------------------------------------------------------------------
function collectStarts( sequence events )

    -- gather the first pitches of each voice

    integer voice, highest
    sequence starts

    -- handle up to 32 voices    
    starts = repeat( 0, 32 )
    
    -- highest voice encountered
    highest = 0

    -- scan events
    for i = 1 to length( events ) do
    
        -- get voice
        voice = events[i][Voice]
        
        -- has this voice been articulated yet?
        if starts[ voice ] = 0 then          
            -- add as first sounding pitch of voice
            starts[voice] = events[i][Pitch]
            -- highest?
            if voice > highest then
                highest = voice
            end if
        end if
    
    end for

    -- eliminate unused voices
    starts = starts[1..highest]

    return starts

end function    



-----------------------------------------------------------------------------
global procedure lexicon( sequence description, sequence measures )

    -- places an entry into the database
    -- lexicon( "chopin maz one 67/3/1 full", {
    --    {{0, 48, 3000, 3, 100}, {0, 76, 1500, 1, 100}, ...

    sequence this, beated, at

    -- get cursor position
    at = get_position()

    -- show the user what's happening
    printf( 1, "%s    ", {description} )
    
    -- reposition cursor
    position( at[1], at[2] )

    -- create an empty record
    this            = NewMeasure
    
    -- populate it
    this[Composer]      = word(1, description)
    this[Style]         = word(2, description)
    this[Form]          = word(3, description)
    this[Source]        = description
    this[CadenceType]   = word(5, description)
    this[Length]        = length( measures )

    -- store measure information
    for i = 1 to length( measures ) do
        
        -- measure number
        this[Number] = i
        
        -- analysis of measure
        beated = breakIntoBeats( measures[i], TicksPerBeat )
        this[Analysis] = analyze( beated )
        
        -- starting notes in measure
        this[Starts] = collectStarts( measures[i] )        
        
        -- events                
        this[Events] = measures[i]
        
        -- place in database
        db = append( db, this )
        
    end for
    

end procedure



-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- querying the database

-----------------------------------------------------------------------------
function getComposer( integer phrase )
    -- return the composer associated with the phrase
    return db[phrase][Composer]
end function                 

-----------------------------------------------------------------------------
function getForm( integer phrase )
    -- return the phrase type of the phrase
    return db[phrase][Form]
end function

-----------------------------------------------------------------------------
function getCadenceType( integer phrase )  
    -- return the cadence of the phrase
    return db[phrase][CadenceType]
end function

-----------------------------------------------------------------------------
function getSource( integer phrase )  
    -- return the source of the phrase
    return db[phrase][Source]
end function

-----------------------------------------------------------------------------
function getLength( integer phrase )  
    -- return the measures phrase
    return db[phrase][Length]
end function

-----------------------------------------------------------------------------
function getNumber( integer measure )
    -- return the measure number
    return db[measure][Number]
end function

-----------------------------------------------------------------------------
function getCadence( integer measure )
    -- return the measure number of the cadence
    -- cadence is the last measure of the phrase
    return measure - getNumber( measure ) + getLength( measure )
end function


-----------------------------------------------------------------------------
function getAnalysis( integer measure )
    -- return the analysis for the measure
    return db[measure][Analysis]
end function

-----------------------------------------------------------------------------
function getStarts( integer measure )
    -- return the starting notes of the measure
    return db[measure][Starts]
end function


-----------------------------------------------------------------------------
function getSeeks( integer measure )
    -- return the seek notes of the measure     
          
    -- last measure?
    if getNumber( measure ) = getLength( measure ) then
        -- no seek notes
        return {}
    else                             
        -- return starting notes of next measure
        return getStarts( measure+1 )        
    end if
    
end function

-----------------------------------------------------------------------------
function getEvents( integer measure )
    -- return the measure number of the measure in the phrase
    return db[measure][Events]
end function

-----------------------------------------------------------------------------
function getEvent( integer measure, integer index )
    -- return the event of the phrase's measure
    return db[measure][Events][index][Time]
end function

-----------------------------------------------------------------------------
function getPitch( integer measure, integer index )
    -- return the pitch of the phrase's measure's indexed event
    return db[measure][Events][index][Pitch]
end function

-----------------------------------------------------------------------------
function getDuration( integer measure, integer index )
    -- return the duration of the phrase's measure's indexed event
    return db[measure][Events][index][Duration]
end function

-----------------------------------------------------------------------------
function getVoice( integer measure, integer index )
    -- return the voice of the phrase's measure's indexed event
    return db[measure][Events][index][Voice]
end function

-----------------------------------------------------------------------------
function getVolume( integer measure, integer index )
    -- return the volume of the phrase's measure's indexed event
    return db[measure][Events][index][Volume]
end function






-----------------------------------------------------------------------------
-- MIDI FILE SUPPORT
-----------------------------------------------------------------------------

sequence midiList
atom ticks

midiList = {}
ticks = 0

-----------------------------------------------------------------------------
procedure midiOn( integer h, integer t, integer ch, integer p )

    printf( h, "%g On  ch=%d n=%d v=64\n", {t,ch,p} )

end procedure


-----------------------------------------------------------------------------
procedure midiOff( integer h, integer t, integer ch, integer p )

    printf( h, "%g Off ch=%d n=%d v=0\n", {t,ch,p} )

end procedure



-----------------------------------------------------------------------------
procedure makeMidiFile( sequence fileName, sequence events )

    -- generate MIDI file
    -- events are in the form { {time, "On"/"Off", pitch} ... }
    
    integer handle, last, tick, pitch, voice, volume
    sequence state

    -- sort the events
    events = sort( events )
    
    -- open the file
    handle = open( fileName, "w" )
    if handle = -1 then
        clear_screen()
        printf( 1, "Unable to open %s\n", {fileName} )
        abort(0)
    end if

    -- midi header
    printf( handle, "MFile 1 %g %g\n", {1+1, TicksPerBeat} )
    puts( handle, "MTrk\n" )
    puts( handle, "0 TimeSig 4/4 24 8\n" )
    puts( handle, "0 KeySig 0 major\n" )
    puts( handle, "0 Tempo 625000\n" )
    puts( handle, "0 Meta TrkEnd\n" )
    puts( handle, "TrkEnd\n" )


    -- build the midi events

    -- midi track
    puts( handle, "MTrk\n" )

    for i = 1 to length( events ) do
        -- get an event
        tick    = events[i][1]
        state   = events[i][2]
        pitch   = events[i][3]
        voice   = events[i][4]
        volume  = events[i][5]
        
        -- generate the midi event
        printf( handle, "%g %s ch=%d n=%d v=%d\n", 
            { tick, state, voice, pitch, volume } )
            
    end for

    -- end file; use tick in last event
    last = length( events )
    printf( handle, "%g Meta TrkEnd\n", {events[last][Time]} )
    puts( handle, "TrkEnd\n" )


    -- close the file
    close( handle )

end procedure


-----------------------------------------------------------------------------
procedure initMidiFile()

    -- initialize midi event list and tick counter
    ticks = 0    
    midiList = {}
end procedure


-----------------------------------------------------------------------------
procedure makeMidiEvents( sequence events, integer beats )

    -- add events to event list for measure of music
    
    integer tick, pitch
    sequence this

    for i = 1 to length( events ) do

        -- get the midi data
        this = events[i]

        -- calculate start time
        tick = floor( this[Time] ) + ticks

        -- change mode?
        if changeMode then
            -- flatten note?
            if find( this[Pitch], FLATTEN ) then
                -- flatten it
                this[Pitch] = this[Pitch] -1
            end if
        end if

        -- note on
        midiList = append( midiList, 
            { tick, "On", this[Pitch], this[Voice], this[Volume] } )
            
        -- calculate end time
        tick = tick + floor( this[Duration] )
                
        -- note off
        midiList = append( midiList, 
            { tick, "Off", this[Pitch], this[Voice], 0 } )

    end for     
    
    -- advance ticks to next measure
    ticks = ticks + (TicksPerBeat*beats)

end procedure


-----------------------------------------------------------------------------
-- LOAD THE DATABASE
-----------------------------------------------------------------------------


-----------------------------------------------------------------------------
-- LOAD THE DATABASE
-----------------------------------------------------------------------------

puts( 1, "Loading: " )

-- load styles
    -- joplin
        -- intros 
        include j-ntr-0.mus
        include j-ntr-1.mus
        include j-ntr-2.mus
        include j-ntr-3.mus
        include j-ntr-4.mus
        include j-ntr-5.mus
        include j-ntr-6.mus
        include j-ntr-7.mus
        include j-ntr-8.mus
        include j-ntr-9.mus
        include j-ntr-r.mus
        include j-ntr-00.mus
        include j-ntr-01.mus
        include j-ntr-02.mus
        include j-ntr-03.mus
        include j-ntr-04.mus
  
        -- phrases
        include jop-1.mus
        include jop-2.mus
        include jop-3.mus
        include jop-4.mus
        include jop-5.mus
        include jop-6.mus
        include jop-7.mus
        include jop-8.mus
        include jop-9.mus
        include jop-r.mus
        include jop-01.mus
        include jop-02.mus
        include jop-03.mus
        include jop-04.mus

    -- chopin mazurkas
        -- phrases
        include chopin1.mus
        include chopin2.mus
        include chopin3.mus
        include chopin4.mus
        include chopin5.mus
        include chopin6.mus
        include chopin7.mus
        include chopin8.mus
        include chopin9.mus
        include chopin10.mus
        include chopin11.mus
        include chopin12.mus
        include chopin13.mus
        include chopin14.mus
        include chopin15.mus
        include chopin16.mus

     -- mozart
        -- phrases
        include mozart1.mus
        include mozart2.mus
        include mozart3.mus
        include mozart4.mus
        include mozart5.mus
        include mozart6.mus
        include mozart7.mus
        include mozart8.mus
        include mozart9.mus
        include mozart10.mus
        include mozart11.mus
        include mozart12.mus
        include mozart13.mus
        include mozart14.mus
        include mozart15.mus

    -- mozart
        -- phrases
        include moz2-1.mus
        include moz2-2.mus
        include moz2-3.mus
        include moz2-4.mus
        include moz2-5.mus
        include moz2-6.mus
        include moz2-7.mus
        include moz2-8.mus
        include moz2-9.mus
        include moz2-10.mus
        include moz2-11.mus
        include moz2-12.mus
        include moz2-13.mus
        include moz2-14.mus
        include moz2-15.mus
  
-- check size
    if length(db) = 0 then
        clear_screen()
        puts( 1, "Error in database!\n" )
        abort(0)
    end if





-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- Composition


constant             
    -- these are the right notes for a full midi major scale
    MAJOR_SCALE = { 36,38,40,41,43,45,47,48,50,52,53,55,57,59,60,62,64,
                      65,67,69,71,72,74,76,77,79,81,83,84,86,88,89,91,93,
                      95,96,98,100,101,103,105,107 },
    MAJOR_SCALE_DOWN = reverse( MAJOR_SCALE )


-----------------------------------------------------------------------------
function isTonal( integer pitch )
    -- returns true if pitch is tonal
    --  ? isTonal( 79 )
    --  --> 1

    -- tonal if is in major notes list    
    return (find( pitch, MAJOR_SCALE ) != 0)
    
end function

-----------------------------------------------------------------------------
function getNeighborTone( integer note )
    -- returns neighor to note
    --  ? getNeighborTone( 79 )
    --  --> 77
    integer at

    -- find position in scale    
    at = find( note, MAJOR_SCALE )

    -- pick upper/lower neighbor
    if at = 1 then
        at = at + 1
    elsif at = length( MAJOR_SCALE ) then
        at = at - 1
    else
        at = at + chooseOne({1,-1}) 
    end if
    
    return MAJOR_SCALE[at]
    
end function

-----------------------------------------------------------------------------
function addNeighborTones( sequence events )
    -- adds neighbor tones to the event list
    --  ? addNeighborTones( {{0,79,1000,1,70},{1000,79,250,1,70}})
    --  --> {{0,79,500,1,70},{500,77,500,1,70},{1000,79,250,1,70}}

    sequence new
    
    -- intialize
    new = {}

    -- for each event
    for i = 1 to length( events ) do
        -- is this followed by a neighbor
        if i < length( events ) then
            -- same pitch, and 1 beat duration      
            if  events[i][Pitch] = events[i+1][Pitch]
            and isTonal( events[i][Pitch] )
            and events[i][Duration] = TicksPerBeat
            then

                -- halve the duration
                events[i][Duration] = TicksPerBeat/2
            
                -- save
                new = append( new, events[i] )
            
                -- replace event with neighbor
                events[i][Pitch] = getNeighborTone(events[i][Pitch])
                
            end if
        end if
        -- save event (or neighbor)
        new = append( new, events[i] )
    end for

    return new
    
end function


-----------------------------------------------------------------------------
function makeEvents( sequence notes, sequence event )
    -- used by add-passing-tones it makes the appropriate events from 
    -- its arguments
    -- ex:  ? makeEvents( {65,67,69}, {0,79,1000,1,70})
    --      --> {{0,65,333,1,70},{333,67,333,1,70},{666,69,333,1,70}}

    integer startTime, duration
    sequence events

    -- get the start time
    startTime = event[Time]                          
    
    -- calculate duration
    duration = floor( (event[Time] + event[Duration]) / length( notes ) )
    
    -- make an event for each note
    events = repeat( event, length( notes ) )
    
    -- replace pitches and timing
    for i = 1 to length( notes ) do
        -- set the event values
        events[i][Time]     = startTime
        events[i][Pitch]    = notes[i]
        events[i][Duration] = duration
        -- increment the time
        startTime = startTime + duration
    end for                             
    
    return events
    
end function

-----------------------------------------------------------------------------
function addTheScaleNotes( integer beginNote, integer endNote )

    -- used by addTheScaleNotes it fills in tonal scales between the 
    -- members of its argument
    -- ex:  ? addTheScaleNotes( 60, 72 )
    --      --> {60,60,62,64,65,67,69,71}
    -- ex:  ? addTheScaleNotes( 72, 60 )
    --      --> {72,71,69,67,65,64,62}
    
    sequence scale
    integer iStart, iEnd

    -- which scale to use?
    if beginNote > endNote then
        -- descending scale
        scale = MAJOR_SCALE_DOWN
    else
        -- ascending scale
        scale = MAJOR_SCALE
    end if
    
    -- get indexes of notes    
    iStart = find( beginNote, scale )
    iEnd = find( endNote, scale )
    
    -- return range, exclusive of end note
    return scale[iStart..iEnd-1]
    
end function


-----------------------------------------------------------------------------
function addPassingTones( sequence events )
    -- used by embellish it adds passing tones between members of its 
    -- argument based given:
    --  initial event has duration of 1000
    --  number of passing tones is even
    --  distance not greater than 4 semitones ( abs(a-b)<=4 )
    --  notes are placed between two notes of scale
    --  there is no rest between the notes
    -- ex:  ? addPassingTones( {{0,79,1000,1,70},{1000,83,250,1,70}})
    --      --> {{0,79,500,1,70},{500,81,500,1,70},{1000,83,250,1,70}}

    integer thisPitch, nextPitch
    sequence new, scale, insert

    -- initialize the new list
    new = {}
    
    for i = 1 to length( events ) do
    
        -- by default, insert current event
        insert = {events[i]}
    
        -- note follows?
        if i < length( events ) then

            -- get pitches under consideration
            thisPitch = events[i][Pitch]
            nextPitch = events[i+1][Pitch]     
            
            -- candidate?
            if  events[i][Duration] = TicksPerBeat
            and isTonal( thisPitch )
            and isTonal( nextPitch )
            and events[i+1][Time] = events[i][Time]+events[i][Duration] 
            then

                -- get the scale notes
                scale = addTheScaleNotes( thisPitch, nextPitch )
                
                -- only create fill if even number of notes (no duples)
                if isEven( length( scale ) ) then
                    insert = makeEvents( scale, events[i] )
                end if
                
            end if
        end if
        
        -- insert the old event, or new events
        new = new & insert
        
    end for
    
    return new
    
end function



-----------------------------------------------------------------------------
function embellish( sequence music )
    -- used in create-work it adds scales and neighboring tones to 
    -- its argument as possible
    -- ex:  ? embellish( {{0,79,1000,1,70},{1000,79,250,1,70}})
    --      --> {{0,79,1000,1,70},{1000,79,250,1,70}}

    if theRecombinance > 75 then
        music = addPassingTones( addNeighborTones(music) )
    end if
    
    return music


end function


-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- Recomposition


-----------------------------------------------------------------------------
procedure addExplanation( sequence text, sequence args )
    -- add text to the explanation
    -- parameters are the same as to sprintf
    -- this is used to explain how the music was recomposed
    explain = append( explain, sprintf( text, args ) )
end procedure


-----------------------------------------------------------------------------
function gatherAllIncipMeasures()

    -- returns indexes of all incipient measures
    
    sequence result

    result = {}
    -- search database
    for measure = 1 to length( db ) do
        -- if not incipient and last measure
        if equal( getForm( measure ), "intro" ) then
            result = append( result, measure )
        end if
    end for

    return result
    
end function

-----------------------------------------------------------------------------
function gatherAllPhraseMeasures()

    -- returns indexes of all phrase measures in the database
    -- excludes incipient and cadence measures

    sequence result

    result = {}
    -- search database
    for measure = 1 to length( db ) do                       
        -- if not incip and not last measure (cadence)
        if not equal( getForm( measure ), "intro" )
        and getNumber( measure ) != getLength( measure ) then
            result = append( result, measure )
        end if
    end for

    return result
    
end function




-----------------------------------------------------------------------------
function gatherAllCadenceMeasures( sequence cadenceType )

    -- return all the cadence measures in the database    

    integer last  
    sequence result

    result = {}
    -- search database
    for measure = 1 to length( db ) do
        -- matches cadence, and end of phrase?
        if equal( getCadenceType( measure ), cadenceType )
        and getNumber( measure ) = getLength( measure ) then
            result = append( result, measure )                    
        end if
    end for

    return result
    
end function



-----------------------------------------------------------------------------
function sieveMeasuresByComposer( sequence list, object composer )

    -- elimitate phrases from list not matching  style
    sequence result

    -- build sublist where style matches
    result = {}
    for i = 1 to length( list ) do          
        -- get indexes
        if equal( getComposer( list[i] ), composer ) then
            -- place in list
            result = append( result, list[i] )
        end if
    end for

    return result
    
end function



-----------------------------------------------------------------------------
function sieveMeasuresByAnalysis( sequence list, sequence analysis )

    -- elimitate options from list not matching analysis
    sequence result, thisAnalysis

    -- build sublist where analysis matches
    result = {}
    for i = 1 to length( list ) do          
        -- get analysis
        thisAnalysis = getAnalysis( list[i] )
        
        -- match?
        if useWeak then
            -- only need to match first function
            if equal( thisAnalysis[1], analysis[1] ) then
                result = append( result, list[i] )
            end if
        else
            -- need to match all functions
            if equal( getAnalysis( list[i] ), analysis ) then
                result = append( result, list[i] )
            end if
        end if
    end for

    return result
    
end function


-----------------------------------------------------------------------------
function sieveMeasuresByStarts( sequence list, integer pitch, integer voice )

    -- elimitate options from list not matching starts
    -- this is passed the start note and the voice to match

    integer diff
    sequence result, myStarts

    -- build sublist where starts matches
    result = {}
    for i = 1 to length( list ) do

        -- get starts of measure
        myStarts = getStarts( list[i] )
-- FIX!!!!!        
        if length( myStarts ) >= voice then
            -- compare starts of first voice
            if abs( myStarts[voice] - pitch ) < matchSemi then
                result = append( result, list[i] )
            end if            
        end if
        
    end for

    return result
    
end function


-----------------------------------------------------------------------------
function sieveMeasuresBySeeks( sequence list, integer pitch, integer voice )

    -- elimitate options from list not matching destinations
    -- this is passed the seek note and the voice to match

    integer diff
    sequence result, mySeeks

    -- build sublist where starts matches
    result = {}
    for i = 1 to length( list ) do

        -- get starts of measure
        mySeeks = getSeeks( list[i] )

        -- if there are any seeks...        
        if length( mySeeks ) >= voice then

            -- compare starts of first voice
            if abs( mySeeks[voice] - pitch ) < matchSemi then
                result = append( result, list[i] )
            end if
            
        end if
        
        
    end for

    return result
    
end function


-----------------------------------------------------------------------------
function notEmpty( sequence s1, sequence s2 )
    -- returns s1 if not empty, else s2
    if length( s1 ) then
        return s1
    else
        return s2
    end if
    
end function


-----------------------------------------------------------------------------
function findSuitableCadence( integer measure, sequence cadenceType )

    -- returns a cadence suitable for the phrase under consideration

    integer cadence, pick
    sequence s, starts
    
    -- get index of the cadence
    cadence = getCadence( measure )
    
    -- gather all the cadences in the database
    s = gatherAllCadenceMeasures( cadenceType )

    -- select only same style
    s = sieveMeasuresByComposer( s, getComposer( measure ) )

    -- select only same analysis as cadence
    s = notEmpty( sieveMeasuresByAnalysis(s, getAnalysis( cadence )), s)

    -- get the starts
    starts = getStarts( cadence )
    
    -- sieve on similar start pitch of the first voice
    s = notEmpty( sieveMeasuresByStarts( s, starts[1], 1 ), s )

    -- pick one randomly
    pick = chooseOne(s)
    
    -- add to explanation
    addExplanation( "    Selected from %s", {getSource( pick )} )
    
    return getEvents( pick )
    
end function


-----------------------------------------------------------------------------
function getChannelEvents( integer measure, integer voice )

    -- extracts only events belonging to requested voice
    -- this is used by the matn code to replace material
    -- for a single voice
    
    sequence events, new
    new = {}
    events = getEvents( measure )
    for i = 1 to length( events ) do
        if events[i][Voice] = voice then
            new = append( new, events[i] )
        end if
    end for
    return new
end function


-----------------------------------------------------------------------------
function regeneratePhrase( sequence title, integer measure, sequence kind )

    -- this is the core composition function. it's passed the index
    -- of the phrase to regenerate. it composes a phrase, attempting
    -- to replace the source material with functionally equivalent
    -- material found elsewhere in the database.

    integer pick
    sequence options, phrase, choices, starts, seeks, events, used

    -- no phrase (ex: incipient)
    if measure = 0 then
        addExplanation( "%s : No material in database", {title} )
        return {}
    end if

    -- explain where source came from
    addExplanation( "%s : %s", {title, getSource( measure )} )

    -- initialize
    phrase = {}
    
    -- track notes that have been used
    used = {}

    -- gather all candidates
    if equal( kind, "intro" ) then
        -- gather incipient measures
        choices = gatherAllIncipMeasures()
    else                            
        -- gather phrase measures
        choices = gatherAllPhraseMeasures()
    end if
    
    -- go through the phrase
    for i = 1 to getLength( measure ) do
         
        -- get the starts and seeks
        starts = getStarts( measure )
        seeks  = getSeeks ( measure )

        -- clear midi event list
        events = {}
        
        for voice = 1 to length( starts ) do
    
            -- select only same composer
            options = sieveMeasuresByComposer( choices, getComposer( measure ) )


            -- select only same analysis of phrase
            options = notEmpty( 
                            sieveMeasuresByAnalysis( 
                                options, 
                                getAnalysis( measure ) ), 
                            options )

            -- remove recently used items
            if not useMatn then                   
                -- remove most recent first
                for j = length( used ) to 1 by -1 do
                    options = notEmpty( remove( used[j], options ), options )
                end for
            end if
    
            -- optional starts    
            if useStart then
                -- select only same start notes
                options = notEmpty( 
                        sieveMeasuresByStarts( options, starts[voice], voice ), 
                        options )
            end if
    
            -- optional seeks
            if useSeek
            and length( seeks ) >= voice then
                -- select only same seek notes
                options = notEmpty( 
                        sieveMeasuresBySeeks( options, seeks[voice], voice ), 
                        options )
            end if

            -- remove self from options?
            options = notEmpty( remove( measure, options ), options )
                
            -- select one
            pick = chooseOne( options )
    
            if useMatn then     
    
                -- add to explanation
                addExplanation( "    Measure %d, voice %d: %s %d", 
                    {i, voice, getSource( pick ), getNumber( pick )} )
    
                -- add only events for current voice
                events = events & getChannelEvents( pick, voice )
            
            else
                -- add to explanation
                addExplanation( "    Measure %d: %s %d", 
                    {i, getSource( pick ), getNumber( pick )} )
                     
                -- add events for all voices
                events = getEvents( pick )
             
                -- track as used
                used = append( used, pick )
                
                -- leave the loop
                exit
                
            end if

            -- what measure follows?
            if useAtn then
                measure = pick+1
            else
                measure = measure+1
            end if
                
        end for
    
        -- add to phrase
        phrase = append( phrase, events )
        
    end for                   
    
    return phrase

end function  


-----------------------------------------------------------------------------    
procedure buildMidiPhrase( sequence phrase, integer source )

    -- generate midi file output for phrase
    -- you need the source so you can figure out how many ticks
    -- the phrase is going to get.
    
    integer beats
    for measure = 1 to length( phrase ) do
        
        -- how many beats in source measure?
        beats = length( getAnalysis( source ) )
        
        -- generate midi events
        makeMidiEvents( phrase[measure], beats )
        
        -- point to next measure
        source = source + 1
        
    end for
    
end procedure



-----------------------------------------------------------------------------    
function attachCadences( integer source, sequence p )

    -- returns two versions of the phrase - one with a half cadence
    -- attached, the other with a full cadence attached

    integer last
    sequence half, full

    -- copy music
    half = p
    full = p

    -- find last measure in the phrase
    last = length( p )

    -- replace the last element in the phrase with a half cadence
    half[last] = findSuitableCadence( source, "half" )           

    -- replace the last element in the phrase with a full cadence
    full[last] = findSuitableCadence( source, "full" )

    -- return the phrase with a half and full cadence
    return {half, full}
    
end function


-----------------------------------------------------------------------------
--
-- User Interface
--

-----------------------------------------------------------------------------
function gatherComposers()

    -- return a list of composer names in the database

    sequence composers

    -- gather list of styles from database
    composers = {}
    for i = 1 to length( db ) do
        if not find( getComposer(i), composers ) then
            composers = append( composers, getComposer(i) )
        end if
    end for

    return composers

end function

-----------------------------------------------------------------------------
function numToText( atom n )

    -- convert a number to text

    return sprintf( "%g", n )
    
end function

-----------------------------------------------------------------------------
function textToNum( sequence s )

    -- convert text to number
    
    s = value( s )
    if s[1] = GET_SUCCESS then
        return s[2]
    else
        return 0
    end if
    
end function



-----------------------------------------------------------------------------
procedure showExplanation()
    
    -- displays how measures were derived in a dialog
    -- also generates a text file of the same information
    
    integer handle
    object result


    -- clear controls
    new_dialog()

    -- create controls
    add_control( List, "Derivation of Measures", 1, 1, 60, 8, explain )

    -- create buttons
    add_button( "OK", 6 )

    -- run dialog
    result = run_dialog( 66, 13, "Explanation of Output" )
    
    -- generate text file
    handle = open( fileName & ".TXT", "w" )
    for i = 1 to length( explain ) do
        puts( handle, explain[i] & '\n' )
    end for

    -- close the file    
    close( handle )
    

end procedure



-----------------------------------------------------------------------------
function pickSource( sequence composer, sequence kind )

    -- return the start measure for a phrase that matches the criteria
    -- the measure is selected at random from the candidates
    -- return 0 if no material is found (ex: intro for chopin)

    sequence source, sources, measures

    -- build list
    sources = {}    
    measures = {}
    for measure = 1 to length( db ) do
        if find( getForm(measure), kind )
        and equal( getComposer( measure ), composer ) then
            -- get the source
            source = getSource( measure )
            -- if not already in the list...
            if not find( source, sources ) then
                -- ... add it to the list
                sources = append( sources, source )
                -- ... and the start measure
                measures = append( measures, measure )
            end if
        end if
    end for

    -- anything in database?
    if length( measures ) then
        -- pick a start measure
        return chooseOne( measures )
    else
        return 0
    end if

end function



-----------------------------------------------------------------------------
procedure getCriteria()

    -- front end dialog, asks user to enter criteria

    sequence result

    -- create new dialog
    new_dialog()

    -- 1: file name
    add_control( Sle, "File Name", 2, 1, 12, 1, "output" )

    -- 3: composer
    add_control( Label, "Composer:", 4, 1, 1, 0, 0 )
    add_control( List, "", 2, 11, 10, 3, gatherComposers() )

    -- 5-7: form
    add_control( Label, "Form:", 8, 1, 1, 0, 0 )
    add_radio( "AA", 8, 11, 0, 0 )
    add_radio( "ABA", 9, 11, 0, 1 )
    add_radio( "ABAB", 10, 11, 0, 0 )

    -- recomposition
    add_control( Label, "Recomposition Parameters", 2, 29, 1, 0, 0 )
    add_control( Label, "", 3, 29, 1, 0, 0 )
    
    -- 10: pattern match                                         
    add_control( Sle, "Match within", 4, 29, 2, 1, numToText(matchSemi) )
    add_control( Label, "semitones", 4, 47, 1, 0, 0 )

    -- 12-16: atn, matn, seek, start, mode
    add_control( Check, "Use ATN (buggy!)", 6, 29, 1, 0, useAtn )
    add_control( Check, "Use Micro-ATN", 7, 29, 1, 0, useMatn )
    add_control( Check, "Use Seek Notes", 8, 29, 1, 0, useSeek )
    add_control( Check, "Use Start Notes", 9, 29, 1, 0, useStart )
    add_control( Check, "Use Only First Function", 10, 29, 1, 0, useWeak )
    add_control( Check, "Change Mode", 11, 29, 1, 0, changeMode )

    -- create buttons
    add_button( "OK", 6 )
    add_button( "Exit", 8 )
    --add_button( "Help", 6 )

    result = run_dialog( 61, 15, VERSION )

    -- quit? (escape key or exit button)
    if equal( result[1], "Cancel" )
    or equal( result[1], "Exit" ) then
        clear_screen()
        reset_font()
        puts( 1, "Goodbye from " & VERSION )
        abort(0)
    else
        result = result[2]
    end if
    
    -- extract values
    fileName    = upper( result[1] )
    composer    = result[3]
    if result[5] then form = "AA"   end if
    if result[6] then form = "ABA"  end if
    if result[7] then form = "ABAB" end if
    matchSemi   = textToNum( result[10] )
    useAtn      = result[12]
    useMatn     = result[13]
    useSeek     = result[14]
    useStart    = result[15]
    useWeak     = result[16]
    changeMode  = result[17]
    
end procedure


-----------------------------------------------------------------------------    
procedure createMusic()

    integer sourceA, sourceB, sourceI, last, at
    sequence a, b, i, ok

    -- clear explanation
    explain = {}

    -- get regeneration criteria
    getCriteria()

    -- get 'a', 'b' and 'incipient' phrase source
    sourceI = pickSource( composer, {"intro"} )
    sourceA = pickSource( composer, {"one", "two"} )
    sourceB = pickSource( composer, {"one", "two"} )


    -- rebuild phrases
    i = regeneratePhrase( "Intro Phrase", sourceI, "intro" )
    a = regeneratePhrase( "A Phrase", sourceA, "phrase" )
    b = regeneratePhrase( "B Phrase", sourceB, "phrase" )

    -- attach cadences
    addExplanation( "A Phrase Cadence Derivation", {} )
    a = attachCadences( sourceA, a )
    
    addExplanation( "B Phrase Cadence Derivation", {} )
    b = attachCadences( sourceB, b )

    -- show explanation
    showExplanation()
    
    -- reconstruction

    -- clear events
    midiList = {}
    ticks = 0

    if equal( form, "AA" ) then
        -- build aa form 
        buildMidiPhrase( i, sourceI )
        buildMidiPhrase( a[1], sourceA )
        buildMidiPhrase( a[2], sourceB )
    
    elsif equal( form, "ABA" ) then
        -- build aba form
        buildMidiPhrase( i, sourceI )
        buildMidiPhrase( a[1], sourceA )
        buildMidiPhrase( b[1], sourceB )
        buildMidiPhrase( b[2], sourceB )
        
    elsif equal( form, "ABAB" ) then
        -- build abab form
        buildMidiPhrase( i, sourceI )
        buildMidiPhrase( a[1], sourceA )
        buildMidiPhrase( b[1], sourceB )
        buildMidiPhrase( a[1], sourceA )
        buildMidiPhrase( b[2], sourceB )
    
    end if
    
    -- need to remove extention?
    at = match( ".MID", fileName )
    if at then                      
        -- remove extention
        fileName = fileName[1..at-1]
    end if

    -- generate text data file
    makeMidiFile( fileName & ".TMP", midiList )
    
    -- convert to midi file
    system( "t2mf  " & fileName & ".TMP " & fileName & ".MID", 2 )
    
    -- delete text data file
    system( "del " & fileName & ".TMP", 2 )

    -- inform user 
    ok = msgbox( OK, "Created MIDI File", 
                     "Created MIDI file '" & fileName & ".MID'" )
    
end procedure

    
-----------------------------------------------------------------------------    


-- colorful background
wrap(0)
bk_color( BLUE )
text_color( CYAN )
for i = 1 to 25 do
    position( i, 1 )
    puts( 1, repeat( 176, 80 ) )
end for

-- loop until cancelled
while 1 do
    createMusic()
end while

